"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const now_1 = require("../../../util/now");
const log_1 = require("../../../util/log");
const readDir_1 = require("../../../util/readDir");
const postDeploy = (projectConfig, config, deployResult, git, deployType) => {
    const deployTypeName = deployType === 'back' ? '后端' : '前端';
    return new Promise((resolve, reject) => {
        if (deployResult === null) {
            log_1.default.error('好像发生了什么错误，并没有部署什么静态文件~');
            return;
        }
        deployResult.splice(0, 1);
        let deployedFilesCount = 0, changedFilesCount = 0;
        deployResult.forEach((deployTypeArr, index) => {
            deployedFilesCount += deployTypeArr.length || 0;
            if (Array.isArray(deployTypeArr) && deployTypeArr.length > 0) {
                deployTypeArr.forEach((file) => {
                    const versionReg = /^(.+)(_\d){2,5}(\.\w{2,5})$/i;
                    if (projectConfig.addNoVersionFile && file && versionReg.test(file.path)) {
                        const newFilePath = file.path.replace(/(_\d){2,5}/, '');
                        deployResult[index].push({
                            path: newFilePath,
                            type: file.type,
                        });
                    }
                    if (file && file.type !== 'constant') {
                        changedFilesCount++;
                    }
                });
            }
        });
        if (deployedFilesCount === 0 || changedFilesCount === 0) {
            log_1.default.error('并没有部署什么静态文件，你怕不是在搞我~');
            return;
        }
        if (!projectConfig.version) {
            resolve();
            return;
        }
        const date = now_1.default('yyyymmdd');
        const version = projectConfig.version || '0.0.0';
        const deployLog = {
            date,
            version,
            commit: git.version,
            result: deployResult,
        };
        let deployLogArr = [];
        const logFileName = '.dophinlog.json';
        let deployPath = '', logPath = '';
        if (deployType === 'back') {
            deployPath = config.backStaticDir;
        }
        if (deployType === 'front') {
            deployPath = config.frontStaticDir;
        }
        logPath = path.resolve(deployPath, logFileName);
        if (!logPath) {
            return;
        }
        if (fs.existsSync(logPath)) {
            deployLogArr = require(logPath);
        }
        deployLogArr.unshift(deployLog);
        log_1.default.info(`[${deployTypeName}]开始合并dophinlog...`);
        const versions = new Set(deployLogArr.map((_log) => _log.version));
        deployLogArr = deployLogArr.filter((_log) => {
            if (versions.has(_log.version)) {
                versions.delete(_log.version);
                return true;
            }
            else {
                return false;
            }
        });
        if (deployLogArr.length > 2 && config.deleteInvalidFiles) {
            log_1.default.info(`[${deployTypeName}]开始根据dophinlog清理无用文件...`);
            const validVersions = projectConfig.deployValidVersions || 99;
            const usefulFiles = new Set();
            deployLogArr.forEach((_log, index) => {
                if (index < validVersions) {
                    for (const files of _log.result) {
                        for (const file of files) {
                            usefulFiles.add(path.resolve(deployPath, file.path));
                        }
                    }
                }
            });
            const filesInDeployPath = readDir_1.default(deployPath);
            const deletedFiles = [];
            filesInDeployPath.forEach((file) => {
                if (!usefulFiles.has(file.path) &&
                    /\/(js|css|images)\b/.test(file.path) &&
                    /\.(js|css|png|jpg|gif|jpeg|)$/i.test(file.name)) {
                    fs.unlinkSync(file.path);
                    deletedFiles.push(file.name);
                }
            });
            if (deletedFiles.length) {
                log_1.default.warn(`[${deployTypeName}]删除文件[${deletedFiles.length}]个: ${deletedFiles.join(', ')}`);
            }
            else {
                log_1.default.info(`[${deployTypeName}]本次更新为增量更新，未删除任何文件`);
            }
        }
        const logFileContent = `${JSON.stringify(deployLogArr)}\n`;
        fs.writeFile(logPath, logFileContent, { encoding: 'utf8' }, (err) => {
            if (err) {
                reject(err);
            }
            else {
                if (!config.deleteInvalidFiles) {
                    log_1.default.warn('[危险,谨慎操作]dophin部署过程可能产生垃圾文件(e.g. chunk[hash])，可在do.config.js添加项目deploy参数deleteInvalidFiles以自动删除垃圾文件。');
                }
                resolve();
            }
        });
    });
};
exports.default = postDeploy;
