"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const dns = require("dns");
const http = require("http");
const log_1 = require("../../../util/log");
const dophin_core_1 = require("dophin-core");
const doRequest = (request, response, host, port) => {
    const options = {
        headers: request.headers,
        hostname: host,
        port: port || '80',
        method: request.method,
        path: request.url,
    };
    const req = http.request(options, (res) => {
        response.writeHead(res.statusCode, res.headers);
        res.pipe(response);
    });
    req.on('error', (err) => {
        response.writeHeader(500, {
            'Content-Type': 'text/plain',
        });
        response.write('Request URL: ' + request.url);
        response.end('\nBreak Host Error: ' + err.message);
    });
    request.pipe(req);
};
const goOnline = (request, response) => {
    const hp = (request.headers.host || '').split(':');
    const host = hp[0];
    dns.resolve4(host, (err, addresses) => {
        if (err) {
            response.writeHeader(404, {
                'Content-Type': 'text/plain',
            });
            response.write('Request URL: ' + request.url);
            response.end('\nBreak Host Error: ' + err.message);
        }
        else {
            doRequest(request, response, addresses[0], hp[1]);
            log_1.default.debug(`转发线上地址: ${addresses[0]}:${hp[1] || 80}${request.url}`);
        }
    });
};
const initRewrite = (rootConfig, rootPath) => {
    const rootDir = rootPath + '';
    const defaultRewriteOption = {
        imageHandle: (req, res, next) => {
            const imgHandleReg = /\/(\w+)\/(\w+)\/([^\?]+)(\?.+)?$/;
            if (imgHandleReg.test(req.originalUrl)) {
                const _match = imgHandleReg.exec(req.originalUrl) || [];
                const _proj = _match[1], _type = _match[2], _filename = _match[3];
                const projects = dophin_core_1.getAllProjects(rootPath + '');
                if (projects.includes(_proj)) {
                    if (_type === 'images') {
                        const imageLocalPath = path.resolve(rootDir, `project/${_proj}/build/${_type}`, _filename);
                        if (fs.existsSync(imageLocalPath)) {
                            log_1.default.info(`${_proj}/${_type}/${_filename} 映射到本地build文件夹`);
                            res.writeHead(200, {
                                'Access-Control-Allow-Origin': '*',
                                'Content-Type': `image/${path.extname(imageLocalPath).replace('.', '')}`,
                                'Cache-Control': 'max-age=0',
                                'Server': 'dophin',
                                'X-Powered-By': 'dophin',
                            });
                            res.end(fs.readFileSync(imageLocalPath));
                            return;
                        }
                    }
                }
            }
            log_1.default.warn(`本地不存在图片${req.originalUrl}, 转发到线上`);
            goOnline(req, res);
        },
        handle: (match, req, res, transform, next) => {
            const _proj = match[1], _type = match[2], _filename = match[4];
            let _path = match[3], realFileName = _filename;
            _path = _path ? _path : '';
            if (Array.isArray(transform) && transform.length > 0) {
                for (const rule of transform) {
                    if ((typeof rule[0] === 'string' && _filename.indexOf(rule[0]) > -1) ||
                        (typeof rule[0] === 'object' && rule[0].test(_filename))) {
                        realFileName = _filename.replace(rule[0], rule[1]);
                        break;
                    }
                }
            }
            let filepath = path.resolve(rootDir, `project/${_proj}/build/${_type}${_path}${realFileName}`);
            if (!fs.existsSync(filepath)) {
                filepath = path.resolve(rootDir, `project/${_proj}/build/${_type}${_path}${_filename}`);
            }
            if (fs.existsSync(filepath)) {
                log_1.default.info(`${_type}${_path}${_filename} 映射到本地build文件夹`);
                let _content = fs.readFileSync(filepath);
                _content = `/**\n * 指向本地文件\n * ${filepath}\n */\n\n\n${_content}`;
                if (_type === 'js') {
                    res.writeHead(200, {
                        'Access-Control-Allow-Origin': '*',
                        'Content-Type': 'application/x-javascript',
                        'Cache-Control': 'max-age=0',
                        'Server': 'dophin',
                        'X-Powered-By': 'dophin',
                    });
                }
                else {
                    res.writeHead(200, {
                        'Access-Control-Allow-Origin': '*',
                        'Content-Type': 'text/css',
                        'Cache-Control': 'max-age=0',
                        'Server': 'dophin',
                        'X-Powered-By': 'dophin',
                    });
                }
                res.end(_content);
            }
            else {
                log_1.default.warn(`本地${_proj}项目不存在${_type}${_path}${_filename}, 转发到线上`);
                goOnline(req, res);
            }
        },
    };
    const rewrite = (req, res, next) => {
        const reWriteConfig = rootConfig && rootConfig.rewrite;
        let _rewrite;
        for (const site of reWriteConfig) {
            if (Array.isArray(site.host) && (site.host.indexOf(req.hostname) > -1)) {
                _rewrite = site;
                break;
            }
            if (req.hostname === site.host) {
                _rewrite = site;
                break;
            }
        }
        const imgReg = /^.+\.(png|jpg|gif|apng|webp|jpeg|bmp|'还有谁')/i;
        if (_rewrite && imgReg.test(req.originalUrl)) {
            defaultRewriteOption.imageHandle(req, res, next);
            return;
        }
        if (_rewrite && _rewrite.reg.test(req.originalUrl)) {
            const match = _rewrite.reg.exec(req.originalUrl);
            defaultRewriteOption.handle(match, req, res, _rewrite.transform, next);
            return;
        }
        else {
            next();
        }
    };
    return rewrite;
};
exports.default = initRewrite;
